import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
//import java.awt.Color;

/**
 * Blok pembuat tetromino.
 * 
 * @author Keenan M. Gebze
 * @version 1.0
 */
public class Block  extends Actor
{
    /**
     * Warna dari blok.
     */
    private Color color;
    /**
     * Bongkahan/Tetromino/salah satu dari anak kelas Tetromino
     * yang membuat blok ini.
     */
    private Tetromino t;

    /**
     * Apakah blok ini sudah pasif?
     */
    private boolean isPassive = false;

    /**
     * Konstruktor tanpa parameter, biar mudah memasukan kedalam 'World'.
     */
    public Block() {
        color = Color.RED;
    }

    /**
     * Konstruktor dengan parameter.Merupakan konstruktor yang digunakan.
     * 
     * @param tetromino 'Blok ini dari bongkahan yang mana?'
     * @param warna warna dari blok ini.
     */
    public Block(Tetromino tetromino, Color warna) {
        t = tetromino;
        color = warna;
    }

    /**
     * Pada saat blok ini masuk ke dunia.
     */
    public void addedToWorld(World w) {
        //membuat gambar sebesar 'blok blok' dunia.
        GreenfootImage image = new GreenfootImage(w.getCellSize(), w.getCellSize());
        //menyiapkan warna dengan warna yang telah diberi di konstruktor.
        image.setColor(color);
        //tuangkan warna!
        image.fill();
        //ganti warna lagi, warna hitam yang lumayan transparan.
        image.setColor(new Color(0,0,0, 30));
        //gambar! keliling kotak.
        image.drawRect(0,0,image.getWidth()-1,image.getHeight()-1);
        //gambar sudah jadi, tinggal dipasang sebagai gambar blok ini.
        setImage(image);
    }

    /**
     * Jika sudah pasif, dan sudah terlalu tinggi maka game kalah!
     */
    public void act() {
        if(isPassive() && getY() < 2) {
            TetrisWorld w = (TetrisWorld) getWorld();
            w.gameLose();
        }
    }

    /**
     * Apakah blok ini menyentuh tanah?
     */
    public boolean isGrounded() {
        Tetromino underT = null;
        if(getY()+1>=getWorld().getHeight()){
            return true;
        }
        try {
            Block bunder = (Block)getOneObjectAtOffset(0,1,Block.class);

            if(bunder == null) {
                return false;
            }else {
                if(bunder.getTetromino().equals(this.getTetromino())) {
                    return false;
                }
                if(bunder.isGrounded() || bunder.getTetromino().isGround() || bunder.isPassive()) {
                    return true;
                }
            }
        }catch(NullPointerException e) {
            return true;
        }catch(StackOverflowError e) {
            return false;
        }
        return false;
    }

    /**
     * Memutar TETROMINOnya, bukan bloknya agar lebih mudah, karena tetrominonya tidak terlihat.
     */
    public void rotate() {
        //ambil tetromino pembuat, lalu menjalankan fungsi nextDir dari tetromino tersebut.
        getTetromino().nextDir();
    }

    /**
     * Mengambil tetromino yang membuat blok ini.
     * 
     * @return Tetromino pembuat blok ini.
     */
    public Tetromino getTetromino() {
        return t;
    }

    /**
     * Membuat blok ini pasif.
     */
    public void passive() {
        isPassive = true;
        t = null;
    }

    /**
     * apakah sudah pasif.
     */
    public boolean isPassive() {
        return isPassive;
    }
}
